<?php

/**
 * Class NSClient
 * @author Calvin Custódio
 */
class NSClient
{

	private $instpath = "/usr/local/opmon/libexec/check_nrpe -H ";

	public function __construct()
	{
		$this->mapper = new \Models\WebServices\Mapper\Mapper();
		include '/usr/local/opmon/etc/config.php';
	}

	public function checkCredentials($data)
	{
		if (! isset($data->globalhostaddress) || empty($data->globalhostaddress)) {
			throw new Exception("Host has not been defined!");
		}

		if (! isset($data->globalport) || empty($data->globalport) ) {
			throw new Exception("Globalport has not been defined!");
		}

		if (! isset($data->globaltimeout) || empty($data->globaltimeout)) {
			throw new Exception("Globaltimeout has not been defined!");
		}

	}

	public function getPath($data, $port){
		$path =  $this->instpath . $data ." -p {$port} -t 60 -u -c CheckWMI -a ShowAll ";
		return $path;
	}

	public function get_process($data)
	{
		$result = array();
		$this->checkCredentials($data);

		$command = $this->getPath($data->globalhostaddress, $data->globalport) .
			"'Query:Running_processes=select name from Win32_Process'";
		$services = shell_exec($command);
		$services = array_slice(explode("Name=",$services),1);

		$uniqueServices = array_unique($services);

		foreach ($uniqueServices as $service) {
			$result[] = array("key" => $service, "value" => $service);
			$this->mapper->addElement(
				$service,
				$service,
				array(
					'process' => $service
				)
			);
		}

		$this->mapper->setComboData($result);

		return $this->mapper->getOutput();
	}

	public function get_service($data)
	{
		$result = array();
		$this->checkCredentials($data);

		$command = $this->getPath($data->globalhostaddress, $data->globalport) .
			"'Query:Running_services=select name from Win32_Service'";
		$services = shell_exec($command);
		$services = array_slice(explode("Name=",$services),1);

		$uniqueServices = array_unique($services);

		foreach ($uniqueServices as $service) {
			$result[] = array("key" => $service, "value" => $service);
			$this->mapper->addElement(
				$service,
				$service,
				array(
					'services' => $service
				)
			);
		}

		$this->mapper->setComboData($result);

		return $this->mapper->getOutput();
	}

	public function get_disk($data)
	{
		$result = array();
		$this->checkCredentials($data);

		$command = $this->getPath($data->globalhostaddress, $data->globalport) .
			"'Query:Disks=select name from Win32_LogicalDisk'";
		$disks = shell_exec($command);
		preg_match_all('/Name=(?<disks>\w:)/', $disks, $matches);
		$disks = $matches['disks'];

		foreach ($disks as &$disk) {
			$result[] = array("key" => $disk, "value" => $disk);
			$this->mapper->addElement(
				"Disk_".$disk,
				$disk,
				array(
					'disks' => $disk
				)
			);
		}

		$this->mapper->setComboData($result);

		return $this->mapper->getOutput();
	}

	public function get_physical_disks($data)
	{
		$result = array();
		$this->checkCredentials($data);

		$command = $this->getPath($data->globalhostaddress, $data->globalport) .
			"\"Query:disks=select name from Win32_PerfRawData_PerfDisk_PhysicalDisk\"";

		$disks = shell_exec($command);
		preg_match_all('/Name=(?<disks>(\d (\w:\ ?)+))/', $disks, $matches);

		foreach ($matches['disks'] as $index => $disk) {
			$result[] = array("key" => "Disk $index", "value" => $disk);
			$this->mapper->addElement(
				"Physical_Disk_" . $disk,
				$disk,
				array(
					'disks' => $disk
				)
			);
		}
		$this->mapper->setComboData($result);

		return $this->mapper->getOutput();
	}

	public function get_processor($data)
	{
		$result = array();
		$coreNumber = array();
		$this->checkCredentials($data);

		$command = $this->getPath($data->globalhostaddress, $data->globalport) .
			"'Query:counters=select NumberOfLogicalProcessors from Win32_ComputerSystem'";
		$output = shell_exec($command);

		preg_match('/(?<NumberOfCores>\d+)/', $output, $matches);

		if (! isset($matches[0])) {
			throw new Exception("Unable to query cores");
		}

		$coreNumber = range(0, $matches[0] - 1);

		foreach ($coreNumber as $core) {
			$c = (string)$core;
			$result[] = array("key" => $c, "value" => $c);
			$this->mapper->addElement(
				"Core_{$c}",
				$c,
				array(
					'processors' => $c
				)
			);
		}

		$this->mapper->setComboData($result);

		return $this->mapper->getOutput();
	}

	public function get_core_counters($data)
	{
		$result = array();
		$counters = array(
			array(
				"key" => "Processor Time",
				"value" => "PercentProcessorTime"
			),
			array(
				"key" => "Interrupts/sec",
				"value" => "InterruptsPersec"
			)
		);

		$this->mapper->setComboData($counters);
		foreach ($data->processors as $core) {
			foreach ($counters as $counter) {
				$this->mapper->addElement(
					"{$counter['key']}_Core_{$core}",
					$counter['key'],
					array(
						'processors' => $core,
						'counters' => $counter['value']
					)
				);

			}
		}

		return $this->mapper->getOutput();
	}

	public function get_disk_performance_counters($data)
	{
		$result = array();
		$this->checkCredentials($data);
		$counters = array(
			array(
				"key" => "Disk Reads per sec",
				"value" => "DiskReadsPersec"
			),
			array(
				"key" => "Percent Idle Time",
				"value" => "PercentIdleTime"
			),
			array(
				"key" => "Current Disk Queue Length",
				"value" => "CurrentDiskQueueLength"
			),
			array(
				"key" => "Disk Writes per sec",
				"value" => "DiskWritesPersec"
			)
		);

		$command = $this->getPath($data->globalhostaddress, $data->globalport) .
			"\"Query:disks=select name from Win32_PerfRawData_PerfDisk_PhysicalDisk\"";
		$disks = shell_exec($command);
		preg_match_all('/Name=(?<disks>(\d (\w:\ ?)+))/', $disks, $matches);
		$this->mapper->setComboData($counters);

		foreach ($matches['disks'] as $disk) {
			foreach ($counters as $counter) {
				$this->mapper->addElement(
					"{$counter['key']}_Disk_{$disk}",
					$counter['key'],
					array(
						'diskperformance' => $disk,
						'diskperformancecounters' => $counter['value']
					)
				);

			}
		}

		return $this->mapper->getOutput();
	}

}
